@@ -186,6 +186,19 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
     if delta is None:
         delta = 0
 
+    if not np.isscalar(delta):
+        delta = np.array(delta)
+
+    if not np.isscalar(zeta):
+        zeta = np.array(zeta)
+
+    # Compute ptau from delta
+    beta0 = particle_ref._xobject.beta0[0]
+    delta_beta0 = delta * beta0
+    ptau_beta0 = (delta_beta0 * delta_beta0
+                        + 2. * delta_beta0 * beta0 + 1.)**0.5 - 1.
+    pzeta = ptau_beta0 / beta0 / beta0
+
     if (x_norm is not None or px_norm is not None
         or y_norm is not None or py_norm is not None):
 
@@ -314,7 +327,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         # Transform long. coordinates to normalized space
         XX_long = np.zeros(shape=(6, num_particles), dtype=np.float64)
         XX_long[4, :] = zeta - particle_on_co.zeta
-        XX_long[5, :] = delta - particle_on_co.delta
+        XX_long[5, :] = pzeta - particle_on_co.ptau / beta0
 
         XX_norm_scaled = np.dot(WWinv, XX_long)
 
@@ -331,7 +344,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         XX[2, :] += particle_on_co.y
         XX[3, :] += particle_on_co.py
         XX[4, :] += particle_on_co.zeta
-        XX[5, :] += particle_on_co.delta
+        XX[5, :] += particle_on_co.ptau / beta0
 
     elif mode == 'set':
 
@@ -348,7 +361,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         XX[2, :] = y
         XX[3, :] = py
         XX[4, :] = zeta
-        XX[5, :] = delta
+        XX[5, :] = pzeta
 
     elif mode == "shift":
 
@@ -365,7 +378,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         XX[2, :] = y + particle_ref.y
         XX[3, :] = py + particle_ref.py
         XX[4, :] = zeta + particle_ref.zeta
-        XX[5, :] = delta + particle_ref.delta
+        XX[5, :] = pzeta + particle_ref.ptau / beta0
     else:
         raise ValueError('What?!')
 
@@ -374,7 +387,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
     part_dict['y'] = XX[2, :]
     part_dict['py'] = XX[3, :]
     part_dict['zeta'] = XX[4, :]
-    part_dict['delta'] = XX[5, :]
+    part_dict['ptau'] = XX[5, :] * beta0
 
     part_dict['weight'] = np.zeros(num_particles, dtype=np.int64)
 
