@@ -136,17 +136,25 @@ class Particles(xo.HybridClass):
             'rvv': '_rvv',
             'rpp': '_rpp'}
 
-    extra_sources = [
-    _pkg_root.joinpath('random_number_generator/rng_src/base_rng.h'),
-    _pkg_root.joinpath('random_number_generator/rng_src/particles_rng.h')]
+    _extra_c_sources = [
+        _pkg_root.joinpath('random_number_generator/rng_src/base_rng.h'),
+        _pkg_root.joinpath('random_number_generator/rng_src/particles_rng.h')]
 
+    _kernels = {
+        'Particles_initialize_rand_gen': xo.Kernel(
+            args=[
+                xo.Arg(xo.ThisClass, name='particles'),
+                xo.Arg(xo.UInt32, pointer=True, name='seeds'),
+                xo.Arg(xo.Int32, name='n_init')],
+            n_threads='n_init')
+        }
 
     _structure = {
             'size_vars': size_vars,
             'scalar_vars': scalar_vars,
             'per_particle_vars': per_particle_vars}
 
-    def __init__(self, **kwargs):
+    def __init__(self,**kwargs):
 
         input_kwargs = kwargs.copy()
 
@@ -249,6 +257,9 @@ class Particles(xo.HybridClass):
                 else:
                     self.reorganize()
 
+    def init_pipeline(self,name):
+        self.name = name
+
     def to_dict(self, copy_to_cpu=True,
                 remove_underscored=None,
                 remove_unused_space=None,
@@ -447,7 +458,7 @@ class Particles(xo.HybridClass):
 
     def _init_random_number_generator(self, seeds=None):
 
-        self.compile_custom_kernels(only_if_needed=True)
+        self.compile_kernels(only_if_needed=True)
 
         if seeds is None:
             seeds = np.random.randint(low=1, high=4e9,
@@ -758,16 +769,6 @@ class Particles(xo.HybridClass):
         raise NotImplementedError('This functionality has been removed')
 
 
-Particles.XoStruct.custom_kernels = {
-    'Particles_initialize_rand_gen': xo.Kernel(
-        args=[
-            xo.Arg(Particles.XoStruct, name='particles'),
-            xo.Arg(xo.UInt32, pointer=True, name='seeds'),
-            xo.Arg(xo.Int32, name='n_init')],
-        n_threads='n_init')}
-
-
-
 def _str_in_list(string, str_list):
 
     found = False
@@ -1038,6 +1039,97 @@ void LocalParticle_update_pzeta(LocalParticle* part, double new_pzeta_value){
     LocalParticle_update_ptau(part, beta0*new_pzeta_value);
 
 }
+
+
+#ifdef XTRACK_GLOBAL_POSLIMIT
+
+/*gpufun*/
+void global_aperture_check(LocalParticle* part0){
+
+
+    //start_per_particle_block (part0->part)
+        double const x = LocalParticle_get_x(part);
+        double const y = LocalParticle_get_y(part);
+
+	int64_t const is_alive = (int64_t)(
+                      (x >= -XTRACK_GLOBAL_POSLIMIT) &&
+		      (x <=  XTRACK_GLOBAL_POSLIMIT) &&
+		      (y >= -XTRACK_GLOBAL_POSLIMIT) &&
+		      (y <=  XTRACK_GLOBAL_POSLIMIT) );
+
+	// I assume that if I am in the function is because
+    	if (!is_alive){
+           LocalParticle_set_state(part, -1);
+	}
+    //end_per_particle_block
+
+
+}
+#endif
+
+/*gpufun*/
+void increment_at_element(LocalParticle* part0){
+
+   //start_per_particle_block (part0->part)
+        LocalParticle_add_to_at_element(part, 1);
+   //end_per_particle_block
+
+
+}
+
+/*gpufun*/
+void increment_at_turn(LocalParticle* part0, int flag_reset_s){
+
+    //start_per_particle_block (part0->part)
+	LocalParticle_add_to_at_turn(part, 1);
+	LocalParticle_set_at_element(part, 0);
+    if (flag_reset_s>0){
+        LocalParticle_set_s(part, 0.);
+    }
+    //end_per_particle_block
+}
+
+
+// check_is_active has different implementation on CPU and GPU
+
+#define CPUIMPLEM //only_for_context cpu_serial cpu_openmp
+
+#ifdef CPUIMPLEM
+
+/*gpufun*/
+int64_t check_is_active(LocalParticle* part) {
+    int64_t ipart=0;
+    while (ipart < part->_num_active_particles){
+        if (part->state[ipart]<1){
+            LocalParticle_exchange(
+                part, ipart, part->_num_active_particles-1);
+            part->_num_active_particles--;
+            part->_num_lost_particles++;
+        }
+	else{
+	    ipart++;
+	}
+    }
+
+    if (part->_num_active_particles==0){
+        return 0;//All particles lost
+    } else {
+        return 1; //Some stable particles are still present
+    }
+}
+
+#else
+
+/*gpufun*/
+int64_t check_is_active(LocalParticle* part) {
+    return LocalParticle_get_state(part)>0;
+};
+
+#endif
+
+#undef CPUIMPLEM //only_for_context cpu_serial cpu_openmp
+
+
 '''
 
     source = '\n\n'.join([src_typedef, src_adders, src_getters,
